@@ -5,6 +5,7 @@
 #  ALL RIGHTS RESERVED.
 #
 #  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
+from typing import Type
 from urllib.parse import urlparse, parse_qs
 
 from prometheus_tools.metrics_registry import MetricsRegistry
@@ -18,25 +19,30 @@ def _get_disc_file_storage(
         location: str, max_workers: str | None = None,
         root_dir: str | None = None,
         metrics: MetricsRegistry | None = None,
+        storage: Type[DiscFileStorage] | None = None,
 ) -> DiscFileStorage:
-    config = DiscFileStorage.Config(
+    storage = storage or DiscFileStorage
+    config = storage.Config(
         location=location, root_dir=root_dir or "", max_workers=max_workers if max_workers is None else int(max_workers)
     )
-    context = DiscFileStorage.Context(metrics=metrics or MetricsRegistry(MetricsRegistry.Config()))
-    return DiscFileStorage(config, context)
+    context = storage.Context(metrics=metrics or MetricsRegistry(MetricsRegistry.Config()))
+    return storage(config, context)
 
 
 def _get_s3_file_storage(access_key: str, secret_key: str, address: str | None = None, region_name: str | None = None,
                          bucket_name: str | None = None, root_dir: str | None = None,
-                         metrics: MetricsRegistry | None = None) -> S3FileStorage:
+                         metrics: MetricsRegistry | None = None,
+                         storage: Type[S3FileStorage] | None = None,
+                         ) -> S3FileStorage:
     kwargs = {"access_key": access_key, "secret_key": secret_key}
     kwargs |= {"address": address} if address is not None else {}
     kwargs |= {"region_name": region_name} if region_name is not None else {}
     kwargs |= {"bucket_name": bucket_name} if bucket_name is not None else {}
     kwargs |= {"root_dir": root_dir} if root_dir is not None else {}
-    config = S3FileStorage.Config(**kwargs)
-    context = S3FileStorage.Context(metrics=metrics or MetricsRegistry(MetricsRegistry.Config()))
-    return S3FileStorage(config, context)
+    storage = storage or S3FileStorage
+    config = storage.Config(**kwargs)
+    context = storage.Context(metrics=metrics or MetricsRegistry(MetricsRegistry.Config()))
+    return storage(config, context)
 
 
 def _pop_get_first(query_params: dict[str, list[str]], key: str) -> str | None:
@@ -52,7 +58,10 @@ S3_URL_FORMAT = '(s3|s3s)://<access_key>:<secret_key>@[<address>][:<port>]/[<reg
 S3_URL_EXAMPLE = 's3://access:secret@localhost:7480/Default?bucket=abm-bucket&root_dir=abm-dir'
 
 
-def get_file_storage(url: str, metrics: MetricsRegistry | None = None) -> AbstractFileStorage:
+def get_file_storage(
+        url: str, metrics: MetricsRegistry | None = None,
+        s3_storage: Type[S3FileStorage] | None = None, disc_storage: Type[DiscFileStorage] | None = None
+) -> AbstractFileStorage:
     parsed_url = urlparse(url)
     query_params: dict[str, list[str]] = parse_qs(parsed_url.query)
     if parsed_url.scheme == "file":
@@ -60,7 +69,7 @@ def get_file_storage(url: str, metrics: MetricsRegistry | None = None) -> Abstra
                                        f"got: {parsed_url.netloc}, format: {S3_URL_FORMAT}, example: {S3_URL_EXAMPLE}")
         result = _get_disc_file_storage(
             location=parsed_url.path, root_dir=_pop_get_first(query_params, 'root_dir'),
-            max_workers=_pop_get_first(query_params, 'max_workers'), metrics=metrics
+            max_workers=_pop_get_first(query_params, 'max_workers'), metrics=metrics, storage=disc_storage
         )
     elif parsed_url.scheme == "s3" or parsed_url.scheme == "s3s":
         assert parsed_url.username, f'Missing access_key, format: {S3_URL_FORMAT}, example: {S3_URL_EXAMPLE}'
@@ -73,7 +82,7 @@ def get_file_storage(url: str, metrics: MetricsRegistry | None = None) -> Abstra
         result = _get_s3_file_storage(
             access_key=parsed_url.username, secret_key=parsed_url.password, address=address, region_name=path or None,
             bucket_name=_pop_get_first(query_params, 'bucket'), root_dir=_pop_get_first(query_params, 'root_dir'),
-            metrics=metrics
+            metrics=metrics, storage=s3_storage
         )
     else:
         raise TypeError(f"Attempt to get storage from unsupported scheme '{parsed_url.scheme}', supported: file|s3|s3s")
